#include <ESP8266WiFi.h>
#include <WiFiUdp.h>

const char ssid[] = "esp8266ap";  //  your network SSID (name)
const char pass[] = "esp8266ap";  // your network password
 
WiFiUDP udp;
unsigned int localPort = 10000;
const int OSC_PACKET_SIZE = 256;
char packetBuffer[OSC_PACKET_SIZE];
int status = WL_IDLE_STATUS;

#include "ps2dev.h"

#define TM_PRESS 1

// スキャンコード(A-Zのキー)
uint8_t kcode[]= {
  0x1c,0x32,0x21,0x23,0x24,0x2b,0x34,0x33,0x43,0x3b,
  0x42,0x4b,0x3a,0x31,0x44,0x4d,0x15,0x2d,0x1b,0x2c,
  0x3c,0x2a,0x1d,0x22,0x35,0x1a
};

// スキャンコード(0-9のキー or '!'-')'の文字)
uint8_t kcode_num[]= {
  0x45,0x16,0x1e,0x26,0x25,0x2e,0x36,0x3d,0x3e,0x46
};

PS2dev keyboard(13,12); 
int enabled =0; 
int code;

void ack() {
  while(keyboard.write(0xFA));
}

int keyboardcommand(int command) {
  unsigned char val;
  switch (command) {
  case 0xFF:  ack();
    while(keyboard.write(0xAA)!=0);
    break;
  case 0xFE: // Reset: キーボードリセットコマンド。正しく受け取った場合ACKを返す。その後キーボードはセルフテストを実行する。
    ack();
    break;
  case 0xF6: // set defaults: 起動時の状態へ戻す
    //enter stream mode
    ack();
    break;
  case 0xF5: //disable data reporting: 起動時の状態へ戻し、キースキャンを停止する
    //FM
    enabled = 0;
    ack();
    break;
  case 0xF4: //enable data reporting : キースキャンを開始する
    //FM
    enabled = 1;
    ack();
    break;
  case 0xF3: //set typematic rate/delay : 
    ack();
    keyboard.read(&val); //do nothing with the rate
    ack();
    break;
  case 0xF2: //get device id : 
    ack();
    keyboard.write(0xAB);
    keyboard.write(0x83);
    break;
  case 0xF0: //set scan code set
    ack();
    keyboard.read(&val); //do nothing with the rate
    ack();
    break;
  case 0xEE: //echo :キーボードが接続されている場合、キーボードはパソコンへ応答（ECHO Responce）を返す。
    //ack();
    keyboard.write(0xEE);
    break;
  case 0xED: //set/reset LEDs :キーボードのLEDの点灯/消灯要求。これに続くオプションバイトでLEDを指定する。 
    ack();
    keyboard.read(&val); //do nothing with the rate
    ack();
    break;
  }
}

void setup() {
  Serial.begin(115200);
  // send the keyboard start up
  while(keyboard.write(0xAA)!=0);

  //Serial.begin(115200);
  while (!Serial) {
    ; // wait for serial port to connect. Needed for Leonardo only
  }

  WiFi.softAP(ssid, pass);
  IPAddress myIP = WiFi.softAPIP();
  Serial.print("AP IP address: ");
  Serial.println(myIP);
 
  Serial.println("Starting UDP");
  udp.begin(localPort);
  Serial.print("Local port: ");
  Serial.println(udp.localPort());
}

void loop() {

  unsigned char c;  // PCからの送信データ
  uint8_t inkey;    // シリアルポートからの取得文字コード
  uint8_t code;     // シーケンスコード 
  uint8_t shifton;  // シフトキーの状態
  uint8_t e0=0;
  
  int rlen = 0;
  while (1) {
    rlen = udp.parsePacket();
    if(rlen<10) {
      delay(1);
      continue;
    }
    break;
  }
    
  code = 0;
  udp.read(packetBuffer, (rlen > OSC_PACKET_SIZE) ? OSC_PACKET_SIZE : rlen);
  //inkey = packetBuffer[1];
  switch((int)packetBuffer[1]+(int)packetBuffer[0]*256) {
    case 1://Left cross up
      inkey = 0x1E;
      break;
    case 2://Left cross down
      inkey = 0x1F;
      break;
    case 4://Left cross right
      inkey = 0x1D;
      break;
    case 8://Left cross left
      inkey = 0x1C;
      break;
    case 16://Y
      inkey = 'Y';
      break;
    case 32://A
      inkey = 'A';
      break;
    case 64://B
      inkey = 'B';
      break;
    case 256://X
      inkey = 'X';
      break;
  }
  Serial.println(inkey,HEX);
  /*
  inkey = Serial.read();
  Serial.write(inkey);
  Serial.write(':');      
  Serial.println(inkey,HEX);
  */
  if ( inkey >='a' && inkey <='z') {
    // a-zの文字
    shifton = 0;
    code = kcode[inkey - 'a'];
  } else if (inkey >='A' && inkey <='Z') {
    // A-Zの文字
    shifton = 1;
    code = kcode[inkey - 'A'];        
  } else if (inkey >='0' && inkey <='9') {
    // 0-9の文字
    shifton = 0;
    code = kcode_num[inkey - '0'];
  } else if (inkey >='!' && inkey <=')') {
    // '!'-')'の文字
    shifton = 1;
    code = kcode_num[inkey - '!'+1];
  } else {
    code = 0;
        shifton = 0;
        e0=0;
        switch(inkey) {
          case 0x2d: code = 0x4e; break; // -
          case 0x5e: code = 0x55; break; // ^
          case 0x5c: code = 0x6a; break; // \
          case 0x40: code = 0x54; break; // @
          case 0x5b: code = 0x5b; break; // [
          case 0x3b: code = 0x4c; break; // ;
          case 0x3a: code = 0x52; break; // :
          case 0x5d: code = 0x5d; break; // ]          
          case 0x2c: code = 0x41; break; // ]
          case 0x2e: code = 0x49; break; // ` 
          case 0x2f: code = 0x4a; break; // /
          //case 0x5c: code = 0x51; break; // \ '
          case 0x20: code = 0x29; break; // \ ' 
          case 0x08: code = 0x66; break; // Bksp
          case 0x0a: code = 0x5a; break; // enter
          case 0x0d: code = 0x5a; break; // enter
          case 0x1b: code = 0x76; break; // ESC

          case 0x1E: code = 0x75; e0=0xE0; break; // up
          case 0x1F: code = 0x72; e0=0xE0; break; // down
          case 0x1C: code = 0x6B; e0=0xE0; break; // left
          case 0x1D: code = 0x74; e0=0xE0; break; // right
    }
    if (!code) {
      shifton = 1;
      switch(inkey) {
          case 0x3d: code = 0x4e; break; // =
          case 0x7e: code = 0x55; break; // ~
          case 0x7c: code = 0x6a; break; // |
          case 0x60: code = 0x54; break; // `
          case 0x7b: code = 0x5b; break; // {
          case 0x2b: code = 0x4c; break; // +
          case 0x2a: code = 0x52; break; // *
          case 0x7d: code = 0x5d; break; // ]
          case 0x3c: code = 0x41; break; // } 
          case 0x3e: code = 0x49; break; // <
          case 0x3f: code = 0x4a; break; // >
          case 0x5f: code = 0x51; break; // _
      }
    }
  }
  // PCに押したキーのシーケンスコードを送信
  if (code) {
    if (shifton) {
      keyboard.write(0x12);  // シフトキーを押した
    } else {
      keyboard.write(0xF0);  // シフトキーを離した 
      keyboard.write(0x12);  
    }
    if(e0) keyboard.write(e0);
    keyboard.write(code);    // キーを押した
    delay(TM_PRESS);
    if(e0) keyboard.write(e0);
    keyboard.write(0xF0);    // キーを離した 
    keyboard.write(code);        
  }
}


